#include "sprites.h"
#include <math.h>

SunSprite::SunSprite(QwSpritePixmapSequence *seq)
      :QwSprite(seq)
{
   z(0);
}


MobileSprite::MobileSprite(QwSpritePixmapSequence *seq, int pn)
      :QwRealMobileSprite(seq)
{
   stopped=false;
   playerNumber=pn;
}

MobileSprite::MobileSprite(int pn)
      :QwRealMobileSprite()
{
   stopped=false;
   playerNumber=pn;
}

void MobileSprite::forward(double mult, int fr)
{
   if(!stopped)
      QwRealMobileSprite::forward(mult,fr);
   else
      frame(fr);
}

void MobileSprite::forward(double mult)
{
   if(!stopped)
      QwRealMobileSprite::forward(mult);
}


ShipSprite::ShipSprite(QwSpritePixmapSequence *seq,int pn)
      :MobileSprite(seq,pn)
{
   hitpoints=99;
   energy=99.9;
   explosion=-1;
   z(-20);
   rotation=0;
}

ShipSprite::ShipSprite(int pn)
      :MobileSprite(pn)
{
   hitpoints=99;
   energy=99.9;
   wins=0;
   explosion=-1;
   z(-20);
   rotation=0;
}

void ShipSprite::setRotation(float r)
{
   int nf,of=frame();
   rotation=r;
   while(rotation<0)
      rotation+=M_PI*2;
   nf=(int)(rotation/(M_PI*2)*ROTNUM)%ROTNUM;
   if(nf!=of)
      frame(nf);
}

void ShipSprite::forward(double mult)
{
   MobileSprite::forward(mult);
   if(explosion>0)
   {
      explosion--;
      if(explosion==0)
         hide();
   }
}

void ShipSprite::forward(double mult,int fr)
{
   MobileSprite::forward(mult,fr);
   rotation=fr/ROTNUM*M_PI*2;
   if(explosion>0)
   {
      explosion--;
      if(explosion==0)
         hide();
   }
}

BulletSprite::BulletSprite(QwSpritePixmapSequence *seq,int pn)
      :MobileSprite(seq,pn)
{
   z(-10);
}

BulletSprite::BulletSprite(int pn)
      :MobileSprite(pn)
{
   z(-10);
}


MineSprite::MineSprite(QwSpritePixmapSequence *seq,int pn,float atime,float f)
      :MobileSprite(seq,pn)
{
   activateTime=atime;
   z(-25);
   fuel=f;
   explosiontime=0;
   timeToGo=0.0;
   expl=false;
   active=false;
}

MineSprite::MineSprite(int pn,float atime,float f)
      :MobileSprite(pn)
{
   activateTime=atime;
   z(-25);
   fuel=f;
   explosiontime=0.0;
   timeToGo=0.0;
   expl=false;
   active=false;
}

void MineSprite::explode(QwSpritePixmapSequence *seq)
{
   setSequence(seq);
   timeToGo=seq->frameCount();
   expl=true;
   frame(0);
   explosiontime=0.0;
   z(-8);
   setFuel(0.0);
}

void MineSprite::forward(double mult)
{
   if(active)
   {
      if(expl)
      {
         explosiontime+=mult;
         if(explosiontime>(timeToGo-0.001))
            explosiontime=timeToGo-0.01;
         frame((int)explosiontime);
      }
   }
   else
   {
      activateTime-=(float)mult;
      if(activateTime<0.0)
      {
         active=true;
         frame(1);
      }
   }
   if(fuel<0.001)
      MobileSprite::forward(mult);
}

float ExplosionSprite::explosionSpeed=1.0;

ExplosionSprite::ExplosionSprite(QwSpritePixmapSequence *seq,MobileSprite *sp)
      :QwRealSprite(seq)
{
   over=false;
   z(-5);
   obj=sp;
   timeToGo=seq->frameCount();
   time=0;

   moveTo(sp->exact_x(),sp->exact_y());
}

ExplosionSprite::ExplosionSprite(MobileSprite *sp)
      :QwRealSprite()
{
   over=false;
   z(-5);
   time=0;
   obj=sp;

   moveTo(sp->exact_x(),sp->exact_y());
}

void ExplosionSprite::forward()
{
   int of=frame();
   moveTo(obj->exact_x(),obj->exact_y());
   time+=explosionSpeed;
   
   if(time>=timeToGo)
   {
      over=true;
      hide();
   }
   else
      if((int)time!=of)
         frame((int)time);
}

void ExplosionSprite::setSequence(QwSpritePixmapSequence *seq)
{
   timeToGo=seq->frameCount();
   QwRealSprite::setSequence(seq);
}
